__app_name__ = "LinkChecker"
__version__ = "10.6.0"
__release_date__ = "2025-07-28"
__copyright_year__ = "2025"
__author__ = "LinkChecker Authors"
__url__ = "https://linkchecker.github.io/linkchecker/"
__support_url__ = "https://github.com/linkchecker/linkchecker/issues"
